/*
**********************************************************************************************************************
*											        eGon
*						           the Embedded GO-ON Bootloader System
*									       eGON arm boot sub-system
*
*						  Copyright(C), 2006-2010, SoftWinners Microelectronic Co., Ltd.
*                                           All Rights Reserved
*
* File    : nand_boot1_write.c
*
* By      : Jerry
*
* Version : V2.00
*
* Date	  :
*
* Descript:
**********************************************************************************************************************
*/
#include "nand_flash.h"

#define TOLERANCE     3         // tolerance for the loop "erase--write--write error--good block by checking"

//#pragma arm section  code="write_in_one_blk"
/*******************************************************************************
*: write_in_one_blk
*ԭͣint32 write_in_one_blk( __u32 blk_num, void *buf,
*                                  __u32 size, __u32 blk_size )
*: ڴдbufʼsizeСblk_numָĿ顣
*ڲ: blk_num           ʵnand flashĿ
*          buf               ʼַ
*          size              дݵĴС
*          blk_size          ʵnand flashĳߴ
*  ֵ: ADV_NF_OK             дɹ
*          ADV_NF_NEW_BAD_BLOCK  ǰ˻
*          ADV_NF_OVERTIME_ERR   ʱ
*    ע: 1. 򲻼鵱ǰĺûĺûڵñǰ
*          2. ӦôӵǰʼַΡŷš
*******************************************************************************/
__s32 write_in_one_blk( __u32 blk_num, void *buf, __u32 size, __u32 blk_size )
{
    __u32 j;
    __s32  status;
    __u32 copy_base;
    __u32 copy_end;
    __u32 blk_end;
    __u32 scts_per_copy;

    for( j = 0, scts_per_copy = size >> NF_SCT_SZ_WIDTH; j < TOLERANCE; j++ )
    {
	    status = NF_erase( blk_num );
	    if( status == NF_OVERTIME_ERR )
	    	return ADV_NF_OVERTIME_ERR;
        else if( status == NF_ERASE_ERR )
        {
        	__wrn("fail in erasing block %u.\n", blk_num);
        	NF_mark_bad_block( blk_num );         // ڱǻǰȲ
        	return ADV_NF_NEW_BAD_BLOCK;
        }
		__msg("Succeed in erasing block %u.\n", blk_num);

        for( copy_base = blk_num * blk_size, copy_end = copy_base + size, blk_end = copy_base + blk_size;
        	 copy_end <= blk_end;
        	 copy_base += size, copy_end = copy_base + size )
        {
    	    status = NF_write( copy_base >> NF_SCT_SZ_WIDTH, buf, scts_per_copy );
    		__msg("finish in progmming address %x on block %u.\n", copy_base, blk_num );
    	    if( status == NF_OVERTIME_ERR )
    	    	return ADV_NF_OVERTIME_ERR;
    	    else if( status == NF_PROG_ERR )
    	    {
	            if( NF_verify_block( blk_num ) == NF_GOOD_BLOCK )
	            	goto try_again;               // 衰--д--д--УǺÿ顱޴TOLERANCEĻ
	            else
					goto failure;		          // ȷʵǻ飬Ȳ
	        }
	    }
	    break;	                                  // ɹ˵ǰ飬ǰѭ

try_again:
		continue;
	}


	if( j < TOLERANCE )                           // ɹ
	{
//	   	__msg("succeed in programming block %u.\n", blk_num);
		return ADV_NF_OK;
	}


failure:
		__msg("fail in programming block %u, it is bad block.\n", blk_num);
	/* Ȳ */
	NF_erase( blk_num );
	NF_mark_bad_block( blk_num );
	return ADV_NF_NEW_BAD_BLOCK;
}
//#pragma arm section  code="write_in_many_blks"
/*******************************************************************************
*: write_in_many_blks
*ԭͣint32 write_in_many_blks( __u32 start_blk, __u32 last_blk_num, void *buf,
*				             	     __u32 size, __u32 blk_size, __u32 * blks )
*: ڴе˳д뵽
*ڲ: start_blk         ʵnand flashʼ
*          last_blk_num      һĿţƷʷΧ
*          buf               ʼַ
*          size              дݵĴС
*          blk_size          ʵnand flashĿС
*          blks              ռݵĿ
*  ֵ: NO_NEW_BAD_BLOCK  дɹû»
*          NF_LACK_BLKS      
*          NF_NEW_BAD_BLOCK  дɹ»
*          OVERTIME_ERR   ʱ
*    ע: ݴʼ鿪ʼΡŷš
*******************************************************************************/
__s32 write_in_many_blks( __u32 start_blk, __u32 last_blk_num, void *buf,
					      __u32 size, __u32 blk_size, __u32 * blks )
{
    __u32 buf_base;
	__u32 buf_off;
    __u32 size_writed;
    __u32 rest_size;
    __u32 blk_num;
    __u32 blk_status;
    __s32 status;


	for( blk_num = start_blk, buf_base = (__u32)buf, buf_off = 0, blk_status = ADV_NF_NO_NEW_BAD_BLOCK;
         blk_num <= last_blk_num && buf_off < size;
         blk_num++ )
    {
    	status = NF_read_status( blk_num );
    	if( status == NF_OVERTIME_ERR )
    		return ADV_NF_OVERTIME_ERR;
    	else if( status == NF_BAD_BLOCK )		            // ǰǻ飬һ
    		continue;

    	rest_size = size - buf_off ;                        // δ벿ֵĳߴ
    	size_writed = ( rest_size < blk_size ) ?  rest_size : blk_size ;  // ȷ˴δĳߴ

    	status = write_in_one_blk( blk_num, (void *)buf_base, size_writed, blk_size );
    	if( status == NF_OVERTIME_ERR )
    		return ADV_NF_OVERTIME_ERR;
    	else if( status == NF_ERASE_ERR )
    	{
    		blk_status = ADV_NF_NEW_BAD_BLOCK;
       		continue;
       	}

    	buf_base += size_writed;
    	buf_off  += size_writed;
    }

    *blks = blk_num - start_blk;                            // ռõĿ飩
    if( buf_off == size )
		return blk_status;                                  // ɹػ
	else
		return ADV_NF_LACK_BLKS;                                // ʧܣΪ鲻
}
/*
************************************************************************************************************
*
*                                             function
*
*    ƣ
*
*    б
*
*    ֵ  
*
*    ˵    
*
*
************************************************************************************************************
*/
__s32  Nand_Burn_Boot1(__u32 Boot1_buf, __u32 length )
{
    __u32 	            i;
    __u32               write_blks;
	__s32               status;
	__s32				ret = -1;

    if(NF_ERROR==NF_open( ))                                             // nand flash
    {
        __inf("%s: NF_open fail !\n",__FUNCTION__);
		return -1;
    }
    if( length <= NF_BLOCK_SIZE )
    {
        for( i = BOOT1_START_BLK_NUM;  i <= BOOT1_LAST_BLK_NUM;  i++ )
        {
            if( NF_read_status( i ) == NF_BAD_BLOCK )		// ǰǻ飬һ
                continue;

            /* ڵǰBoot1ı */
            status = write_in_one_blk( i, (void *)Boot1_buf, length, NF_BLOCK_SIZE );
            if( status == ADV_NF_OVERTIME_ERR )
                goto over_time;
        }
    }
    else
    {
    	for( i = BOOT1_START_BLK_NUM;  i <= BOOT1_LAST_BLK_NUM;  i++ )
        {
            if( NF_read_status( i ) == NF_BAD_BLOCK )		// ǰǻ飬һ
                continue;

        	status = write_in_many_blks( i, BOOT1_LAST_BLK_NUM, (void*)Boot1_buf,
            	                        length, NF_BLOCK_SIZE, &write_blks );
            if( status == ADV_NF_OVERTIME_ERR )
                goto over_time;

        	i = i + write_blks -1;
        }
	}

	ret = 0;
over_time:

    NF_close( );

    return ret;
}








